==> FASTBuild-Src-v0.53.zip: dist_v0.53/Code/Tools/FBuild/FBuildCore/BFF/Functions/Function.h <==
// Function
//------------------------------------------------------------------------------
#pragma once
#ifndef FBUILD_FUNCTIONS_FUNCTION_H
#define FBUILD_FUNCTIONS_FUNCTION_H

// includes
//------------------------------------------------------------------------------
#include <Core/Containers/Array.h>
#include <Core/Strings/AString.h>
#include <Tools/FBuild/FBuildCore/Error.h>

// Forward Declarations
//------------------------------------------------------------------------------
class BFFIterator;
class BFFVariable;
class Node;

// Function
//------------------------------------------------------------------------------
class Function
{
public:
	explicit	Function( const char * functionName );
	virtual		~Function();

	// info about a function
	inline const AString & GetName() const { return m_Name; }

	// access to functions
	static const Function * Find( const AString & name );

	static void Create();
	static void Destroy();

	// does this function support a header? i.e. FunctionName( ...header.... )
	virtual bool AcceptsHeader() const; // can have a header
	virtual bool NeedsHeader() const;	// must have a header

	// must this function be unique?
	virtual bool IsUnique() const;
	inline bool GetSeen() const { return m_Seen; }
	inline void SetSeen() const { m_Seen = true; }

	// most functions don't need to override this
	virtual bool ParseFunction( const BFFIterator & functionNameStart,
								const BFFIterator & functionBodyStartToken, 
								const BFFIterator & functionBodyStopToken,
								const BFFIterator * functionHeaderStartToken,
								const BFFIterator * functionHeaderStopToken ) const;

	// most functions will override this to commit the effects of the function
	virtual bool Commit( const BFFIterator & funcStartIter ) const;

private:
	Function *	m_NextFunction;
	static Function * s_FirstFunction;

protected:
	AString		m_Name;
	mutable bool m_Seen; // track for unique enforcement

	// for functions that support a simple alias parameter, the base class can
	// parse it out
	mutable AString m_AliasForFunction;

	bool GetString( const BFFIterator & iter, const BFFVariable * & var, const char * name, bool required = false ) const;
	bool GetBool( const BFFIterator & iter, bool & var, const char * name, bool defaultValue, bool required = false ) const;
	bool GetNodeList( const BFFIterator & iter, const char * name, Array< Node * > & nodes, bool required = false ) const;

	// helper function to make alias for target
	bool ProcessAlias( const BFFIterator & iter, Node * nodeToAlias ) const;
private:
	bool GetNodeListRecurse( const BFFIterator & iter, const char * name, Array< Node * > & nodes, const AString & nodeName ) const;
};

//------------------------------------------------------------------------------
#endif // FBUILD_FUNCTIONS_FUNCTION_H

==> FASTBuild-Src-v0.58.zip: dist_v0.58/Code/Tools/FBuild/FBuildCore/BFF/Functions/Function.h <==
// Function
//------------------------------------------------------------------------------
#pragma once
#ifndef FBUILD_FUNCTIONS_FUNCTION_H
#define FBUILD_FUNCTIONS_FUNCTION_H

// includes
//------------------------------------------------------------------------------
#include "Core/Containers/Array.h"
#include "Core/Strings/AString.h"
#include "Tools/FBuild/FBuildCore/Error.h"

// Forward Declarations
//------------------------------------------------------------------------------
class BFFIterator;
class BFFVariable;
class Node;

// Function
//------------------------------------------------------------------------------
class Function
{
public:
	explicit	Function( const char * functionName );
	virtual		~Function();

	// info about a function
	inline const AString & GetName() const { return m_Name; }

	// access to functions
	static const Function * Find( const AString & name );

	static void Create();
	static void Destroy();

	// does this function support a header? i.e. FunctionName( ...header.... )
	virtual bool AcceptsHeader() const; // can have a header
	virtual bool NeedsHeader() const;	// must have a header
	virtual bool NeedsBody() const;		// must have a body i.e. { ... }

	// must this function be unique?
	virtual bool IsUnique() const;
	inline bool GetSeen() const { return m_Seen; }
	inline void SetSeen() const { m_Seen = true; }

	// most functions don't need to override this
	virtual bool ParseFunction( const BFFIterator & functionNameStart,
								const BFFIterator * functionBodyStartToken, 
								const BFFIterator * functionBodyStopToken,
								const BFFIterator * functionHeaderStartToken,
								const BFFIterator * functionHeaderStopToken ) const;

	// most functions will override this to commit the effects of the function
	virtual bool Commit( const BFFIterator & funcStartIter ) const;

private:
	Function *	m_NextFunction;
	static Function * s_FirstFunction;

protected:
	AString		m_Name;
	mutable bool m_Seen; // track for unique enforcement

	// for functions that support a simple alias parameter, the base class can
	// parse it out
	mutable AString m_AliasForFunction;

	// helpers to get properties
	bool GetString( const BFFIterator & iter, const BFFVariable * & var, const char * name, bool required = false ) const;
	bool GetStringOrArrayOfStrings( const BFFIterator & iter, const BFFVariable * & var, const char * name, bool required ) const;
	bool GetBool( const BFFIterator & iter, bool & var, const char * name, bool defaultValue, bool required = false ) const;
	bool GetInt( const BFFIterator & iter, int32_t & var, const char * name, int32_t defaultValue, bool required ) const;
	bool GetNodeList( const BFFIterator & iter, const char * name, Array< Node * > & nodes, bool required = false ) const;
	bool GetStrings( const BFFIterator & iter, Array< AString > & strings, const char * name, bool required = false ) const;

	// helpers to clean/fixup paths to files and folders
	void CleanFolderPaths( Array< AString > & folders ) const;
	void CleanFilePaths( Array< AString > & files ) const;

	// helper function to make alias for target
	bool ProcessAlias( const BFFIterator & iter, Node * nodeToAlias ) const;
	bool ProcessAlias( const BFFIterator & iter, Array< Node * > & nodesToAlias ) const;
private:
	bool GetNodeListRecurse( const BFFIterator & iter, const char * name, Array< Node * > & nodes, const AString & nodeName ) const;
};

//------------------------------------------------------------------------------
#endif // FBUILD_FUNCTIONS_FUNCTION_H
